@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2015 Leandro Nini
+ * Copyright 2011-2020 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,20 +47,30 @@
 #include "utils.h"
 #include "ini/dataParser.h"
 
-void debug(const TCHAR *msg)
+#include "sidcxx11.h"
+
+inline void debug(const TCHAR *msg, const TCHAR *val)
 {
 #ifndef NDEBUG
-    SID_COUT << msg << std::endl;
+    SID_COUT << msg << val << std::endl;
 #endif
 }
 
-const TCHAR *IniConfig::DIR_NAME  = TEXT("sidplayfp");
-const TCHAR *IniConfig::FILE_NAME = TEXT("sidplayfp.ini");
+inline void error(const TCHAR *msg)
+{
+    SID_CERR << msg << std::endl;
+}
 
-#define SAFE_FREE(p) { if(p) { free (p); (p)=NULL; } }
+inline void error(const TCHAR *msg, const TCHAR *val)
+{
+    SID_CERR << msg << val << std::endl;
+}
 
-IniConfig::IniConfig() :
-    status(true)
+const TCHAR *DIR_NAME = TEXT("sidplayfp");
+const TCHAR *FILE_NAME = TEXT("sidplayfp.ini");
+
+
+IniConfig::IniConfig()
 {   // Initialise everything else
     clear();
 }
@@ -100,23 +110,40 @@ void IniConfig::clear()
     emulation_s.modelForced   = false;
     emulation_s.sidModel      = SidConfig::MOS6581;
     emulation_s.forceModel    = false;
+    emulation_s.ciaModel      = SidConfig::MOS6526;
     emulation_s.filter        = true;
     emulation_s.engine.clear();
 
-    emulation_s.bias            = 0.0;
-    emulation_s.filterCurve6581 = 0.0;
-    emulation_s.filterCurve8580 = 0;
+    emulation_s.bias            = 0.5;
+    emulation_s.filterCurve6581 = 0.5;
+    emulation_s.filterCurve8580 = 0.5;
 }
 
 
-bool IniConfig::readDouble(iniHandler &ini, const TCHAR *key, double &result)
+// static helpers
+
+const TCHAR* readKey(iniHandler &ini, const TCHAR *key)
 {
     const TCHAR* value = ini.getValue(key);
-    if (value == 0)
+    if (value == nullptr)
     {   // Doesn't exist, add it
         ini.addValue(key, TEXT(""));
-        return false;
+        debug(TEXT("Key doesn't exist: "), key);
     }
+    else if (!value[0])
+    {
+        // Ignore empty values
+        return nullptr;
+    }
+
+    return value;
+}
+
+void readDouble(iniHandler &ini, const TCHAR *key, double &result)
+{
+    const TCHAR* value = readKey(ini, key);
+    if (value == nullptr)
+        return;
 
     try
     {
@@ -124,21 +151,16 @@ bool IniConfig::readDouble(iniHandler &ini, const TCHAR *key, double &result)
     }
     catch (dataParser::parseError const &e)
     {
-        return false;
+        error(TEXT("Error parsing double at "), key);
     }
-
-    return true;
 }
 
 
-bool IniConfig::readInt(iniHandler &ini, const TCHAR *key, int &result)
+void readInt(iniHandler &ini, const TCHAR *key, int &result)
 {
-    const TCHAR* value = ini.getValue(key);
-    if (value == 0)
-    {   // Doesn't exist, add it
-        ini.addValue(key, TEXT(""));
-        return false;
-    }
+    const TCHAR* value = readKey(ini, key);
+    if (value == nullptr)
+        return;
 
     try
     {
@@ -146,35 +168,16 @@ bool IniConfig::readInt(iniHandler &ini, const TCHAR *key, int &result)
     }
     catch (dataParser::parseError const &e)
     {
-        return false;
+        error(TEXT("Error parsing int at "), key);
     }
-
-    return true;
 }
 
 
-bool IniConfig::readString(iniHandler &ini, const TCHAR *key, SID_STRING &result)
+void readBool(iniHandler &ini, const TCHAR *key, bool &result)
 {
-    const TCHAR* value = ini.getValue(key);
-    if (value == 0)
-    {   // Doesn't exist, add it
-        ini.addValue(key, TEXT(""));
-        return false;
-    }
-
-    result.assign(value);
-    return true;
-}
-
-
-bool IniConfig::readBool(iniHandler &ini, const TCHAR *key, bool &result)
-{
-    const TCHAR* value = ini.getValue(key);
-    if (value == 0)
-    {   // Doesn't exist, add it
-        ini.addValue(key, TEXT(""));
-        return false;
-    }
+    const TCHAR* value = readKey(ini, key);
+    if (value == nullptr)
+        return;
 
     try
     {
@@ -182,19 +185,31 @@ bool IniConfig::readBool(iniHandler &ini, const TCHAR *key, bool &result)
     }
     catch (dataParser::parseError const &e)
     {
-        return false;
+        error(TEXT("Error parsing bool at "), key);
     }
-
-    return true;
 }
 
 
-bool IniConfig::readChar(iniHandler &ini, const TCHAR *key, char &ch)
+SID_STRING readString(iniHandler &ini, const TCHAR *key)
 {
-    SID_STRING str;
-    bool ret = readString(ini, key, str);
-    if (!ret)
-        return false;
+    const TCHAR* value = ini.getValue(key);
+    if (value == nullptr)
+    {
+        // Doesn't exist, add it
+        ini.addValue(key, TEXT(""));
+        debug(TEXT("Key doesn't exist: "), key);
+        return SID_STRING();
+    }
+
+    return SID_STRING(value);
+}
+
+
+void readChar(iniHandler &ini, const TCHAR *key, char &ch)
+{
+    SID_STRING str = readString(ini, key);
+    if (str.empty())
+        return;
 
     TCHAR c = 0;
 
@@ -202,7 +217,7 @@ bool IniConfig::readChar(iniHandler &ini, const TCHAR *key, char &ch)
     if (str[0] == '\'')
     {
         if (str[2] != '\'')
-            ret = false;
+            return;
         else
             c = str[1];
     } // Nope is number
@@ -214,23 +229,20 @@ bool IniConfig::readChar(iniHandler &ini, const TCHAR *key, char &ch)
         }
         catch (dataParser::parseError const &e)
         {
-            return false;
+            error(TEXT("Error parsing int at "), key);
         }
     }
 
     // Clip off special characters
     if ((unsigned) c >= 32)
         ch = c;
-
-    return ret;
 }
 
 
-bool IniConfig::readTime(iniHandler &ini, const TCHAR *key, int &value)
+bool readTime(iniHandler &ini, const TCHAR *key, int &value)
 {
-    SID_STRING str;
-    bool ret = readString(ini, key, str);
-    if (!ret)
+    SID_STRING str = readString(ini, key);
+    if (str.empty())
         return false;
 
     int time;
@@ -256,30 +268,30 @@ bool IniConfig::readTime(iniHandler &ini, const TCHAR *key, int &value)
     }
     catch (dataParser::parseError const &e)
     {
+        error(TEXT("Error parsing time at "), key);
         return false;
     }
 
     value = time;
-    return ret;
+    return true;
 
 IniCofig_readTime_error:
+    error(TEXT("Invalid time at "), key);
     return false;
 }
 
 
-bool IniConfig::readSidplay2(iniHandler &ini)
+void IniConfig::readSidplay2(iniHandler &ini)
 {
     if (!ini.setSection(TEXT("SIDPlayfp")))
         ini.addSection(TEXT("SIDPlayfp"));
 
-    bool ret = true;
-
     int version = sidplay2_s.version;
-    ret &= readInt (ini, TEXT("Version"), version);
+    readInt(ini, TEXT("Version"), version);
     if (version > 0)
         sidplay2_s.version = version;
 
-    ret &= readString(ini, TEXT("Songlength Database"), sidplay2_s.database);
+    sidplay2_s.database = readString(ini, TEXT("Songlength Database"));
 
 #if !defined _WIN32 && defined HAVE_UNISTD_H
     if (sidplay2_s.database.empty())
@@ -292,79 +304,68 @@ bool IniConfig::readSidplay2(iniHandler &ini)
 #endif
 
     int time;
-    if (readTime (ini, TEXT("Default Play Length"), time))
-        sidplay2_s.playLength   = (uint_least32_t) time;
-    if (readTime (ini, TEXT("Default Record Length"), time))
-        sidplay2_s.recordLength = (uint_least32_t) time;
+    if (readTime(ini, TEXT("Default Play Length"), time))
+        sidplay2_s.playLength = time;
+    if (readTime(ini, TEXT("Default Record Length"), time))
+        sidplay2_s.recordLength = time;
 
-    ret &= readString(ini, TEXT("Kernal Rom"), sidplay2_s.kernalRom);
-    ret &= readString(ini, TEXT("Basic Rom"), sidplay2_s.basicRom);
-    ret &= readString(ini, TEXT("Chargen Rom"), sidplay2_s.chargenRom);
-
-    return ret;
+    sidplay2_s.kernalRom = readString(ini, TEXT("Kernal Rom"));
+    sidplay2_s.basicRom = readString(ini, TEXT("Basic Rom"));
+    sidplay2_s.chargenRom = readString(ini, TEXT("Chargen Rom"));
 }
 
 
-bool IniConfig::readConsole(iniHandler &ini)
+void IniConfig::readConsole(iniHandler &ini)
 {
     if (!ini.setSection (TEXT("Console")))
         ini.addSection(TEXT("Console"));
 
-    bool ret = true;
-
-    ret &= readBool (ini, TEXT("Ansi"),                console_s.ansi);
-    ret &= readChar (ini, TEXT("Char Top Left"),       console_s.topLeft);
-    ret &= readChar (ini, TEXT("Char Top Right"),      console_s.topRight);
-    ret &= readChar (ini, TEXT("Char Bottom Left"),    console_s.bottomLeft);
-    ret &= readChar (ini, TEXT("Char Bottom Right"),   console_s.bottomRight);
-    ret &= readChar (ini, TEXT("Char Vertical"),       console_s.vertical);
-    ret &= readChar (ini, TEXT("Char Horizontal"),     console_s.horizontal);
-    ret &= readChar (ini, TEXT("Char Junction Left"),  console_s.junctionLeft);
-    ret &= readChar (ini, TEXT("Char Junction Right"), console_s.junctionRight);
-    return ret;
+    readBool(ini, TEXT("Ansi"),                console_s.ansi);
+    readChar(ini, TEXT("Char Top Left"),       console_s.topLeft);
+    readChar(ini, TEXT("Char Top Right"),      console_s.topRight);
+    readChar(ini, TEXT("Char Bottom Left"),    console_s.bottomLeft);
+    readChar(ini, TEXT("Char Bottom Right"),   console_s.bottomRight);
+    readChar(ini, TEXT("Char Vertical"),       console_s.vertical);
+    readChar(ini, TEXT("Char Horizontal"),     console_s.horizontal);
+    readChar(ini, TEXT("Char Junction Left"),  console_s.junctionLeft);
+    readChar(ini, TEXT("Char Junction Right"), console_s.junctionRight);
 }
 
 
-bool IniConfig::readAudio(iniHandler &ini)
+void IniConfig::readAudio(iniHandler &ini)
 {
     if (!ini.setSection (TEXT("Audio")))
         ini.addSection(TEXT("Audio"));
 
-    bool ret = true;
-
     {
         int frequency = (int) audio_s.frequency;
-        ret &= readInt (ini, TEXT("Frequency"), frequency);
+        readInt(ini, TEXT("Frequency"), frequency);
         audio_s.frequency = (unsigned long) frequency;
     }
 
     {
         int channels = 0;
-        ret &= readInt (ini, TEXT("Channels"),  channels);
+        readInt(ini, TEXT("Channels"),  channels);
         if (channels)
         {
             audio_s.playback = (channels == 1) ? SidConfig::MONO : SidConfig::STEREO;
         }
     }
 
-    ret &= readInt (ini, TEXT("BitsPerSample"), audio_s.precision);
-    return ret;
+    readInt(ini, TEXT("BitsPerSample"), audio_s.precision);
 }
 
 
-bool IniConfig::readEmulation(iniHandler &ini)
+void IniConfig::readEmulation(iniHandler &ini)
 {
     if (!ini.setSection (TEXT("Emulation")))
         ini.addSection(TEXT("Emulation"));
 
-    bool ret = true;
-
-    ret &= readString (ini, TEXT("Engine"), emulation_s.engine);
+    emulation_s.engine = readString (ini, TEXT("Engine"));
 
     {
-        SID_STRING str;
-        const bool res = readString (ini, TEXT("C64Model"), str);
-        if (res)
+        SID_STRING str = readString (ini, TEXT("C64Model"));
+        if (!str.empty())
         {
             if (str.compare(TEXT("PAL")) == 0)
                 emulation_s.modelDefault = SidConfig::PAL;
@@ -375,40 +376,89 @@ bool IniConfig::readEmulation(iniHandler &ini)
             else if (str.compare(TEXT("DREAN")) == 0)
                 emulation_s.modelDefault = SidConfig::DREAN;
         }
-        ret &= res;
     }
 
-    ret &= readBool (ini, TEXT("ForceC64Model"), emulation_s.modelForced);
+    readBool(ini, TEXT("ForceC64Model"), emulation_s.modelForced);
+    readBool(ini, TEXT("DigiBoost"), emulation_s.digiboost);
 
     {
-        SID_STRING str;
-        const bool res = readString (ini, TEXT("SidModel"), str);
-        if (res)
+        SID_STRING str = readString(ini, TEXT("CiaModel"));
+        if (!str.empty())
+        {
+            if (str.compare(TEXT("MOS6526")) == 0)
+                emulation_s.ciaModel = SidConfig::MOS6526;
+            else if (str.compare(TEXT("MOS8521")) == 0)
+                emulation_s.ciaModel = SidConfig::MOS8521;
+        }
+    }
+
+    {
+        SID_STRING str = readString(ini, TEXT("SidModel"));
+        if (!str.empty())
         {
             if (str.compare(TEXT("MOS6581")) == 0)
                 emulation_s.sidModel = SidConfig::MOS6581;
             else if (str.compare(TEXT("MOS8580")) == 0)
                 emulation_s.sidModel = SidConfig::MOS8580;
         }
-        ret &= res;
     }
 
-    ret &= readBool (ini, TEXT("ForceSidModel"), emulation_s.forceModel);
+    readBool(ini, TEXT("ForceSidModel"), emulation_s.forceModel);
 
-    ret &= readBool (ini, TEXT("UseFilter"), emulation_s.filter);
+    readBool(ini, TEXT("UseFilter"), emulation_s.filter);
 
-    ret &= readDouble (ini, TEXT("FilterBias"), emulation_s.bias);
-    ret &= readDouble (ini, TEXT("FilterCurve6581"), emulation_s.filterCurve6581);
-    ret &= readInt (ini, TEXT("FilterCurve8580"), emulation_s.filterCurve8580);
-
-    return ret;
+    readDouble(ini, TEXT("FilterBias"), emulation_s.bias);
+    readDouble(ini, TEXT("FilterCurve6581"), emulation_s.filterCurve6581);
+    readDouble(ini, TEXT("FilterCurve8580"), emulation_s.filterCurve8580);
 }
 
-void IniConfig::read()
+class iniError
 {
-    clear();
-    status = false;
+private:
+    const SID_STRING msg;
 
+public:
+    iniError(const TCHAR* msg) : msg(msg) {}
+    const SID_STRING message() const { return msg; }
+};
+
+void createDir(const SID_STRING& path)
+{
+#ifndef _WIN32
+	DIR *dir = opendir(path.c_str());
+    if (dir)
+	{
+		closedir(dir);
+	}
+	else if (errno == ENOENT)
+	{
+        if (mkdir(path.c_str(), 0755) < 0)
+        {
+            throw iniError(strerror(errno));
+        }
+    }
+	else
+	{
+		throw iniError(strerror(errno));
+	}
+#else
+    if (GetFileAttributes(path.c_str()) == INVALID_FILE_ATTRIBUTES)
+    {
+        if (CreateDirectory(path.c_str(), NULL) == 0)
+        {
+            LPTSTR pBuffer;
+            FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
+                NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)&pBuffer, 0, NULL);
+            iniError err(pBuffer);
+            LocalFree(pBuffer);
+            throw err;
+        }
+    }
+#endif
+}
+
+SID_STRING getConfigPath()
+{
     SID_STRING configPath;
 
     try
@@ -417,54 +467,67 @@ void IniConfig::read()
     }
     catch (utils::error const &e)
     {
-        debug(TEXT("Cannot get config path!"));
-        return;
+        throw iniError(TEXT("Cannot get config path!"));
     }
 
+    debug(TEXT("Config path: "), configPath.c_str());
+
+    // Make sure the config path exists
+	createDir(configPath);
+
     configPath.append(SEPARATOR).append(DIR_NAME);
 
-    // Make sure the config path exists
-#ifndef _WIN32
-    if (!opendir(configPath.c_str()))
-    {
-        if (mkdir(configPath.c_str(), 0755) < 0)
-        {
-            debug(strerror(errno));
-            return;
-        }
-    }
-#else
-    if (GetFileAttributes(configPath.c_str()) == INVALID_FILE_ATTRIBUTES)
-    {
-        if (CreateDirectory(configPath.c_str(), NULL) == 0)
-        {
-            LPTSTR pBuffer;
-            FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
-                NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)&pBuffer, 0, NULL);
-            debug(pBuffer);
-            LocalFree(pBuffer);
-            return;
-        }
-    }
-#endif
+    // Make sure the app config path exists
+	createDir(configPath);
 
     configPath.append(SEPARATOR).append(FILE_NAME);
 
-    //debug(configPath.c_str());
+    debug(TEXT("Config file: "), configPath.c_str());
+
+    return configPath;
+}
+
+bool tryOpen(iniHandler &ini)
+{
+#ifdef _WIN32
+    {
+        // Try exec dir first
+        SID_STRING execPath(utils::getExecPath());
+        execPath.append(SEPARATOR).append(FILE_NAME);
+        if (ini.open(execPath.c_str()))
+            return true;
+    }
+#endif
+    return false;
+}
+
+void IniConfig::read()
+{
+    clear();
 
     iniHandler ini;
 
-    // Opens an existing file or creates a new one
-    if (!ini.open(configPath.c_str()))
+    if (!tryOpen(ini))
     {
-        debug(TEXT("Error reading config file!"));
-        return;
+        try
+        {
+            SID_STRING configPath = getConfigPath();
+
+            // Opens an existing file or creates a new one
+            if (!ini.open(configPath.c_str()))
+            {
+                error(TEXT("Error reading config file!"));
+                return;
+            }
+        } catch (iniError const &e) {
+            error(e.message().c_str());
+            return;
+        }
     }
 
-    status = true;
-    status &= readSidplay2  (ini);
-    status &= readConsole   (ini);
-    status &= readAudio     (ini);
-    status &= readEmulation (ini);
+    readSidplay2  (ini);
+    readConsole   (ini);
+    readAudio     (ini);
+    readEmulation (ini);
     ini.close();
 }
