@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2015 Leandro Nini
+ * Copyright 2011-2019 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,9 @@
 #include <climits>
 #include <cstdlib>
 
-//#include "ini/types.h"
+#include "ini/types.h"
+
+#include "sidcxx11.h"
 
 using std::cout;
 using std::cerr;
@@ -37,6 +39,11 @@ using std::endl;
 #  include <sidplayfp/builders/hardsid.h>
 #endif
 
+#ifdef HAVE_SIDPLAYFP_BUILDERS_EXSID_H
+#  include <sidplayfp/builders/exsid.h>
+#endif
+
+// Wide-chars are not yet supported here
 #undef SEPARATOR
 #define SEPARATOR "/"
 
@@ -61,16 +68,16 @@ bool ConsolePlayer::tryOpenTune(const char *hvscBase)
 /**
  * Try load songlength DB from HVSC_BASE
  */
-bool ConsolePlayer::tryOpenDatabase(const char *hvscBase)
+bool ConsolePlayer::tryOpenDatabase(const char *hvscBase, const char *suffix)
 {
     std::string newFileName(hvscBase);
 
-    newFileName.append(SEPARATOR).append("DOCUMENTS").append(SEPARATOR).append("Songlengths.txt");
+    newFileName.append(SEPARATOR).append("DOCUMENTS").append(SEPARATOR).append("Songlengths.").append(suffix);
     return m_database.open(newFileName.c_str());
 }
 
 // Convert time from integer
-bool ConsolePlayer::parseTime (const char *str, uint_least32_t &time)
+bool parseTime(const char *str, uint_least32_t &time)
 {
     // Check for empty string
     if (*str == '\0')
@@ -85,6 +92,7 @@ bool ConsolePlayer::parseTime (const char *str, uint_least32_t &time)
     }
     else
     {   // Read in MM:SS format
+        // TODO parse milliseconds too
         int val;
         *sep = '\0';
         val  = atoi (str);
@@ -97,11 +105,11 @@ bool ConsolePlayer::parseTime (const char *str, uint_least32_t &time)
         _time += (uint_least32_t) val;
     }
 
-    time = _time;
+    time = _time * 1000;
     return true;
 }
 
-bool ConsolePlayer::parseAddress (const char *str, uint_least16_t &address)
+bool parseAddress(const char *str, uint_least16_t &address)
 {
     if (*str == '\0')
         return false;
@@ -112,8 +120,20 @@ bool ConsolePlayer::parseAddress (const char *str, uint_least16_t &address)
     return true;
 }
 
+void displayDebugArgs()
+{
+    std::ostream &out = cout;
+
+    out << "Debug Options:" << endl
+        << " --cpu-debug   display cpu register and assembly dumps" << endl
+        << " --delay=<num> simulate c64 power on delay" << endl
+        << " --noaudio     no audio output device" << endl
+        << " --nosid       no sid emulation" << endl
+        << " --none        no audio output device and no sid emulation" << endl;
+}
+
 // Parse command line arguments
-int ConsolePlayer::args (int argc, const char *argv[])
+int ConsolePlayer::args(int argc, const char *argv[])
 {
     if (argc == 0) // at least one argument required
     {
@@ -124,6 +144,7 @@ int ConsolePlayer::args (int argc, const char *argv[])
     // default arg options
     m_driver.output = OUT_SOUNDCARD;
     m_driver.file   = false;
+    m_driver.info   = false;
 
     for (int i=0; i<9; i++)
     {
@@ -135,7 +156,7 @@ int ConsolePlayer::args (int argc, const char *argv[])
     bool err    = false;
 
     // parse command line arguments
-    while ((i < argc) && (argv[i] != NULL))
+    while ((i < argc) && (argv[i] != nullptr))
     {
         if ((argv[i][0] == '-') && (argv[i][1] != '\0'))
         {
@@ -291,6 +312,10 @@ int ConsolePlayer::args (int argc, const char *argv[])
             {
                 m_engCfg.defaultSidModel = SidConfig::MOS8580;
             }
+            else if (strcmp (&argv[i][1], "-digiboost") == 0)
+            {
+                m_engCfg.digiBoost = true;
+            }
 
             // Video/Verbose Options
             else if (strcmp (&argv[i][1], "vnf") == 0)
@@ -326,7 +351,6 @@ int ConsolePlayer::args (int argc, const char *argv[])
             {
                 m_engCfg.powerOnDelay = (uint_least16_t) atoi(&argv[i][8]);
             }
-
             // File format conversions
             else if (argv[i][1] == 'w')
             {
@@ -342,7 +366,6 @@ int ConsolePlayer::args (int argc, const char *argv[])
                 if (argv[i][5] != '\0')
                     m_outfile = &argv[i][5];
             }
-#ifdef AU_OUTPUT
             else if (strncmp (&argv[i][1], "-au", 3) == 0)
             {
                 m_driver.output = OUT_AU;
@@ -350,7 +373,10 @@ int ConsolePlayer::args (int argc, const char *argv[])
                 if (argv[i][4] != '\0')
                     m_outfile = &argv[i][4];
             }
-#endif
+            else if (strncmp (&argv[i][1], "-info", 5) == 0)
+            {
+                m_driver.info   = true;
+            }
 #ifdef HAVE_SIDPLAYFP_BUILDERS_RESIDFP_H
             else if (strcmp (&argv[i][1], "-residfp") == 0)
             {
@@ -374,6 +400,14 @@ int ConsolePlayer::args (int argc, const char *argv[])
             }
 #endif // HAVE_SIDPLAYFP_BUILDERS_HARDSID_H
 
+#ifdef HAVE_SIDPLAYFP_BUILDERS_EXSID_H
+            else if (strcmp (&argv[i][1], "-exsid") == 0)
+            {
+                m_driver.sid    = EMU_EXSID;
+                m_driver.output = OUT_NULL;
+            }
+#endif // HAVE_SIDPLAYFP_BUILDERS_EXSID_H
+
             // These are for debug
             else if (strcmp (&argv[i][1], "-none") == 0)
             {
@@ -434,7 +468,7 @@ int ConsolePlayer::args (int argc, const char *argv[])
     }
 
     // If filename specified we can only convert one song
-    if (m_outfile != NULL)
+    if (m_outfile != nullptr)
         m_track.single = true;
 
     // Can only loop if not creating audio files
@@ -449,6 +483,11 @@ int ConsolePlayer::args (int argc, const char *argv[])
         return -1;
     }
 
+    if (m_driver.info && m_driver.file)
+    {
+        displayError ("WARNING: metadata can be added only to wav files");
+    }
+
     // Select the desired track
     m_track.first    = m_tune.selectSong (m_track.first);
     m_track.selected = m_track.first;
@@ -469,7 +508,22 @@ int ConsolePlayer::args (int argc, const char *argv[])
                 ? (m_iniCfg.sidplay2()).recordLength
                 : (m_iniCfg.sidplay2()).playLength;
 
-            if (!hvscBase || !tryOpenDatabase(hvscBase))
+
+            bool dbOpened = false;
+            if (hvscBase)
+            {
+                if (tryOpenDatabase(hvscBase, "md5"))
+                {
+                    dbOpened = true;
+                    newSonglengthDB = true;
+                }
+                else if (tryOpenDatabase(hvscBase, "txt"))
+                {
+                    dbOpened = true;
+                }
+            }
+
+            if (!dbOpened)
             {
                 // Try load user configured songlength DB
 #if defined(_WIN32) && defined(UNICODE)
@@ -488,6 +542,9 @@ int ConsolePlayer::args (int argc, const char *argv[])
                         displayError (m_database.error ());
                         return -1;
                     }
+
+                    if (strstr(database, ".md5") != nullptr)
+                        newSonglengthDB = true;
                 }
             }
         }
@@ -540,7 +597,7 @@ void ConsolePlayer::displayArgs (const char *arg)
         << " -o<l|s>      looping and/or single track" << endl
         << " -o<num>      start track (default: preset)" << endl
 
-        << " -p<num>      set format for wav output (16 = signed 16 bit, 32 = 32 bit float)"
+        << " -p<num>      set format for file output (16 = signed 16 bit, 32 = 32 bit float)"
         << "(default: 16)" << endl
 
 #if !defined(DISALLOW_STEREO_SOUND)
@@ -555,14 +612,14 @@ void ConsolePlayer::displayArgs (const char *arg)
 
         << " -m<o|n>[f]   set SID new/old chip model (default: old)" << endl
         << "              Use 'f' to force the model" << endl
+        << " --digiboost  Enable digiboost for 8580 model" << endl
 
         << " -r[i|r][f]   set resampling method (default: resample interpolate)" << endl
         << "              Use 'f' to enable fast resampling (only for reSID)" << endl
 
-        << " -w[name]     create wav file (default: <datafile>[n].wav)" << endl;
-#ifdef AU_OUTPUT
-        << " -au[name]    create au file (default: <datafile>[n].au)" << endl;
-#endif
+        << " -w[name]     create wav file (default: <datafile>[n].wav)" << endl
+        << " --au[name]   create au file (default: <datafile>[n].au)" << endl
+        << " --info       add metadata to wav file" << endl;
 
 #ifdef HAVE_SIDPLAYFP_BUILDERS_RESIDFP_H
     out << " --residfp    use reSIDfp emulation (default)" << endl;
@@ -578,21 +635,14 @@ void ConsolePlayer::displayArgs (const char *arg)
         if (hs.availDevices ())
             out << " --hardsid    enable hardsid support" << endl;
     }
+#endif
+#ifdef HAVE_SIDPLAYFP_BUILDERS_EXSID_H
+    {
+        exSIDBuilder hs("");
+        if (hs.availDevices ())
+            out << " --exsid      enable exSID support" << endl;
+    }
 #endif
     out << endl
         << "Home Page: " PACKAGE_URL << endl;
 }
-
-
-void ConsolePlayer::displayDebugArgs ()
-{
-    std::ostream &out = cout;
-
-    out << "Debug Options:" << endl
-        << " --cpu-debug   display cpu register and assembly dumps" << endl
-        << " --delay=<num> simulate c64 power on delay" << endl
-
-        << " --noaudio     no audio output device" << endl
-        << " --nosid       no sid emulation" << endl
-        << " --none        no audio output device and no sid emulation" << endl;
-}
