@@ -65,11 +65,11 @@ public:  // --------------------------------------------------------- public
     Audio_OSS();
     ~Audio_OSS();
 
-    bool open  (AudioConfig &cfg);
-    void close ();
-    void reset ();
-    bool write ();
-    void pause () {}
+    bool open  (AudioConfig &cfg) override;
+    void close () override;
+    void reset () override;
+    bool write () override;
+    void pause () override {}
 };
 
 #endif // HAVE_*_SOUNDCARD_H
