@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a SID player.
  *
- * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2018 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  * Copyright 2000-2004 Simon White
  * Copyright 2000 Michael Schwendt
@@ -28,6 +28,8 @@
 #include <fstream>
 #include <new>
 
+#include <cstring>
+
 // Get the lo byte (8 bit) in a dword (32 bit)
 inline uint8_t endian_32lo8 (uint_least32_t dword)
 {
@@ -76,20 +78,54 @@ inline void endian_little32 (uint8_t ptr[4], uint_least32_t dword)
     ptr[3] = endian_16hi8  (word);
 }
 
-const wavHeader WavFile::defaultWavHdr = {
-    // ASCII keywords are hex-ified.
-    {0x52,0x49,0x46,0x46}, {0,0,0,0}, {0x57,0x41,0x56,0x45},
-    {0x66,0x6d,0x74,0x20}, {16,0,0,0},
-    {1,0}, {0,0}, {0,0,0,0}, {0,0,0,0}, {0,0}, {0,0},
-    {0x64,0x61,0x74,0x61}, {0,0,0,0}
+const riffHeader WavFile::defaultRiffHdr =
+{
+    // ASCII keywords are hexified.
+    {0x52,0x49,0x46,0x46}, // 'RIFF'
+    {0,0,0,0},             // length
+    {0x57,0x41,0x56,0x45}, // 'WAVE'
+};
+
+const wavHeader WavFile::defaultWavHdr =
+{
+    {0x66,0x6d,0x74,0x20}, // 'fmt '
+    {16,0,0,0},            // length
+    {1,0},                 // AudioFormat (PCM)
+    {0,0},                 // Channels
+    {0,0,0,0},             // Samplerate
+    {0,0,0,0},             // ByteRate
+    {0,0},                 // BlockAlign
+    {0,0},                 // BitsPerSample
+    {0x64,0x61,0x74,0x61}, // 'data'
+    {0,0,0,0}              // length
+};
+
+const listInfo WavFile::defaultListInfo =
+{
+    // ASCII keywords are hexified.
+    {0x4C,0x49,0x53,0x54}, // 'LIST'
+    {124,0,0,0},           // length
+    {0x49,0x4E,0x46,0x4F}, // 'INFO'
+    {0x49,0x4E,0x41,0x4D}, // 'INAM'
+    {32,0,0,0},            // length
+    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
+    {0x49,0x41,0x52,0x54}, // 'IART'
+    {32,0,0,0},            // length
+    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
+    {0x49,0x43,0x4F,0x50}, // 'ICOP'
+    {32,0,0,0},            // length
+    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
 };
 
 WavFile::WavFile(const std::string &name) :
     AudioBase("WAVFILE"),
     name(name),
+    riffHdr(defaultRiffHdr),
     wavHdr(defaultWavHdr),
-    file(0),
+    listHdr(defaultListInfo),
+    file(nullptr),
     headerWritten(false),
+    hasListInfo(false),
     precision(32)
 {}
 
@@ -98,7 +134,7 @@ bool WavFile::open(AudioConfig &cfg)
     precision = cfg.precision;
 
     unsigned short bits       = precision;
-    unsigned short format     = (precision == 16 ) ? 1 : 3;
+    unsigned short format     = (precision == 16) ? 1 : 3;
     unsigned short channels   = cfg.channels;
     unsigned long  freq       = cfg.frequency;
     unsigned short blockAlign = (bits>>3)*channels;
@@ -111,7 +147,7 @@ bool WavFile::open(AudioConfig &cfg)
     if (file && !file->fail())
         close();
 
-    byteCount = 0;
+    dataSize = 0;
 
     // We need to make a buffer for the user
     try
@@ -125,7 +161,7 @@ bool WavFile::open(AudioConfig &cfg)
     }
 
     // Fill in header with parameters and expected file size.
-    endian_little32(wavHdr.length, sizeof(wavHeader)-8);
+    endian_little32(riffHdr.length, sizeof(riffHeader)+sizeof(wavHeader)-8);
     endian_little16(wavHdr.channels, channels);
     endian_little16(wavHdr.format, format);
     endian_little32(wavHdr.sampleFreq, freq);
@@ -154,7 +190,10 @@ bool WavFile::write()
         unsigned long int bytes = _settings.bufSize;
         if (!headerWritten)
         {
-            file->write((char*)&wavHdr,sizeof(wavHeader));
+            file->write((char*)&riffHdr, sizeof(riffHeader));
+            if (hasListInfo)
+                file->write((char*)&listHdr, sizeof(listInfo));
+            file->write((char*)&wavHdr, sizeof(wavHeader));
             headerWritten = true;
         }
 
@@ -168,14 +207,14 @@ bool WavFile::write()
         {
             std::vector<float> buffer(_settings.bufSize);
             bytes *= 4;
-            for (unsigned long i=0;i<_settings.bufSize;i++)
+            // normalize floats
+            for (unsigned long i=0; i<_settings.bufSize; i++)
             {
                 buffer[i] = ((float)_sampleBuffer[i])/32768.f;
             }
             file->write((char*)&buffer.front(), bytes);
         }
-        byteCount += bytes;
-
+        dataSize += bytes;
     }
     return true;
 }
@@ -184,15 +223,30 @@ void WavFile::close()
 {
     if (file && !file->fail())
     {
-        endian_little32(wavHdr.length, byteCount+sizeof(wavHeader)-8);
-        endian_little32(wavHdr.dataChunkLen, byteCount);
+        // update length fields in header
+        unsigned long int headerSize = sizeof(riffHeader)+sizeof(wavHeader)-8;
+        if (hasListInfo)
+            headerSize += sizeof(listInfo);
+        endian_little32(riffHdr.length, headerSize+dataSize);
+        endian_little32(wavHdr.dataChunkLen, dataSize);
         if (file != &std::cout)
         {
             file->seekp(0, std::ios::beg);
+            file->write((char*)&riffHdr, sizeof(riffHeader));
+            if (hasListInfo)
+                file->write((char*)&listHdr, sizeof(listInfo));
             file->write((char*)&wavHdr, sizeof(wavHeader));
             delete file;
         }
-        file = 0;
+        file = nullptr;
         delete[] _sampleBuffer;
     }
 }
+
+void WavFile::setInfo(const char* title, const char* author, const char* released)
+{
+    hasListInfo = true;
+    memcpy(listHdr.name, title, 32);
+    memcpy(listHdr.artist, author, 32);
+    memcpy(listHdr.released, released, 32);
+}
