@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2015 Leandro Nini
+ * Copyright 2011-2019 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,6 +41,53 @@ using std::string;
 #include <sidplayfp/SidInfo.h>
 #include <sidplayfp/SidTuneInfo.h>
 
+const char SID6581[] = "MOS6581";
+const char SID8580[] = "CSG8580";
+
+const char* getModel(SidTuneInfo::model_t model)
+{
+    switch (model)
+    {
+    default:
+    case SidTuneInfo::SIDMODEL_UNKNOWN:
+        return "UNKNOWN";
+    case SidTuneInfo::SIDMODEL_6581:
+        return SID6581;
+    case SidTuneInfo::SIDMODEL_8580:
+        return SID8580;
+    case SidTuneInfo::SIDMODEL_ANY:
+        return "ANY";
+    }
+}
+
+const char* getModel(SidConfig::sid_model_t model)
+{
+    switch (model)
+    {
+    default:
+    case SidConfig::MOS6581:
+        return SID6581;
+    case SidConfig::MOS8580:
+        return SID8580;
+    }
+}
+
+const char* getClock(SidTuneInfo::clock_t clock)
+{
+    switch (clock)
+    {
+    default:
+    case SidTuneInfo::CLOCK_UNKNOWN:
+        return "UNKNOWN";
+    case SidTuneInfo::CLOCK_PAL:
+        return "PAL";
+    case SidTuneInfo::CLOCK_NTSC:
+        return "NTSC";
+    case SidTuneInfo::CLOCK_ANY:
+        return "ANY";
+    }
+}
+
 
 // Display console menu
 void ConsolePlayer::menu ()
@@ -181,7 +228,7 @@ void ConsolePlayer::menu ()
         consoleColour (green, true);
         cerr << " Song Speed   : ";
         consoleColour (white, true);
-        cerr << info.speedString() << endl;
+        cerr << getClock(tuneInfo->clockSpeed()) << endl;
     }
 
     consoleTable  (tableMiddle);
@@ -189,16 +236,20 @@ void ConsolePlayer::menu ()
     cerr << " Song Length  : ";
     consoleColour (white, true);
     if (m_timer.stop)
-        cerr << setw(2) << setfill('0') << ((m_timer.stop / 60) % 100)
-             << ':' << setw(2) << setfill('0') << (m_timer.stop % 60);
+    {
+        const uint_least32_t seconds = m_timer.stop / 1000;
+        cerr << setw(2) << setfill('0') << ((seconds / 60) % 100)
+             << ':' << setw(2) << setfill('0') << (seconds % 60);
+    }
     else if (m_timer.valid)
         cerr << "FOREVER";
     else
         cerr << "UNKNOWN";
     if (m_timer.start)
     {   // Show offset
-        cerr << " (+" << setw(2) << setfill('0') << ((m_timer.start / 60) % 100)
-             << ':' << setw(2) << setfill('0') << (m_timer.start % 60) << ")";
+        const uint_least32_t seconds = m_timer.start / 1000;
+        cerr << " (+" << setw(2) << setfill('0') << ((seconds / 60) % 100)
+             << ':' << setw(2) << setfill('0') << (seconds % 60) << ")";
     }
     cerr << endl;
 
@@ -242,9 +293,7 @@ void ConsolePlayer::menu ()
         consoleColour (yellow, true);
         cerr << " SID Details  : ";
         consoleColour (white, false);
-        cerr << "Filter = "
-             << ((m_filter.enabled == true) ? "Yes" : "No");
-        cerr << ", Model = ";
+        cerr << "Model = ";
 #if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
         cerr << getModel(tuneInfo->sidModel(0));
 #else
@@ -283,6 +332,42 @@ void ConsolePlayer::menu ()
 #endif
         }
 
+        consoleTable  (tableSeparator);
+
+        consoleTable  (tableMiddle);
+        consoleColour (yellow, true);
+        cerr << " Play speed   : ";
+        consoleColour (white, false);
+        cerr << info.speedString() << endl;
+
+        consoleTable  (tableMiddle);
+        consoleColour (yellow, true);
+        cerr << " Play mode    : ";
+        consoleColour (white, false);
+        cerr << (info.channels() == 1 ? "Mono" : "Stereo") << endl;
+
+        consoleTable  (tableMiddle);
+        consoleColour (yellow, true);
+        cerr << " SID Filter   : ";
+        consoleColour (white, false);
+        cerr << (m_filter.enabled ? "Yes" : "No") << endl;
+
+        consoleTable  (tableMiddle);
+        consoleColour (yellow, true);
+        cerr << " DigiBoost    : ";
+        consoleColour (white, false);
+        cerr << (m_engCfg.digiBoost ? "Yes" : "No") << endl;
+
+        consoleTable  (tableMiddle);
+        consoleColour (yellow, true);
+        cerr << " SID Model    : ";
+        consoleColour (white, false);
+        if (m_engCfg.forceSidModel)
+            cerr << "Forced ";
+        else
+            cerr << "from tune, default = ";
+        cerr << getModel(m_engCfg.defaultSidModel) << endl;
+
         if (m_verboseLevel > 1)
         {
             consoleTable  (tableMiddle);
@@ -351,7 +436,7 @@ void ConsolePlayer::menu ()
     if (m_driver.file)
         cerr << "Creating audio file, please wait...";
     else
-        cerr << "Playing, press ^C to stop...";
+        cerr << "Playing, press ESC to stop...";
 
     // Get all the text to the screen so music playback
     // is not disturbed.
@@ -430,18 +515,3 @@ void ConsolePlayer::consoleRestore ()
     if ((m_iniCfg.console ()).ansi)
         cerr << '\x1b' << "[0m";
 }
-
-const char* ConsolePlayer::getModel (SidTuneInfo::model_t model)
-{
-    switch (model)
-    {
-    case SidTuneInfo::SIDMODEL_UNKNOWN:
-        return "UNKNOWN";
-    case SidTuneInfo::SIDMODEL_6581:
-        return "6581";
-    case SidTuneInfo::SIDMODEL_8580:
-        return "8580";
-    case SidTuneInfo::SIDMODEL_ANY:
-        return "ANY";
-    }
-}
