@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2015 Leandro Nini
+ * Copyright 2011-2019 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -68,7 +68,7 @@ typedef enum
     /* The following require a soundcard */
     EMU_DEFAULT, EMU_RESIDFP, EMU_RESID,
     /* The following should disable the soundcard */
-    EMU_HARDSID, EMU_SIDSTATION, EMU_COMMODORE,
+    EMU_HARDSID, EMU_EXSID, EMU_SIDSTATION, EMU_COMMODORE,
     EMU_SIDSYN, EMU_END} SIDEMUS;
 
 typedef enum
@@ -106,6 +106,9 @@ private:
 #ifdef HAVE_SIDPLAYFP_BUILDERS_HARDSID_H
     static const char  HARDSID_ID[];
 #endif
+#ifdef HAVE_SIDPLAYFP_BUILDERS_EXSID_H
+    static const char  EXSID_ID[];
+#endif
 #ifdef HAVE_TSID
     TSID               m_tsid;
 #endif
@@ -127,6 +130,8 @@ private:
 
     bool               m_cpudebug;
 
+    bool               newSonglengthDB;
+
     bool vMute[9];
 
     int  m_precision;
@@ -137,7 +142,7 @@ private:
         double         bias;
         // Filter parameters for reSIDfp
         double         filterCurve6581;
-        int            filterCurve8580;
+        double         filterCurve8580;
 
         bool           enabled;
     } m_filter;
@@ -147,6 +152,7 @@ private:
         OUTPUTS        output;   // Selected output type
         SIDEMUS        sid;      // Sid emulation
         bool           file;     // File based driver
+        bool           info;     // File metadata
         AudioConfig    cfg;
         IAudio*        selected; // Selected Output Driver
         IAudio*        device;   // HW/File Driver
@@ -160,6 +166,7 @@ private:
         uint_least32_t stop;
         uint_least32_t length;
         bool           valid;
+        bool           starting;
     } m_timer;
 
     struct m_track_t
@@ -184,10 +191,7 @@ private:
     void consoleRestore (void);
 
     // Command line args
-    bool parseTime        (const char *str, uint_least32_t &time);
-    bool parseAddress      (const char *str, uint_least16_t &address);
-    void displayArgs      (const char *arg = NULL);
-    void displayDebugArgs ();
+    void displayArgs    (const char *arg = NULL);
 
     bool createOutput   (OUTPUTS driver, const SidTuneInfo *tuneInfo);
     bool createSidEmu   (SIDEMUS emu);
@@ -198,23 +202,22 @@ private:
     void emuflush       (void);
     void menu           (void);
 
-    const char* getModel (SidTuneInfo::model_t model);
-
-    IAudio* getWavFile(const SidTuneInfo *tuneInfo);
+    std::string getFileName(const SidTuneInfo *tuneInfo);
 
     inline bool tryOpenTune(const char *hvscBase);
-    inline bool tryOpenDatabase(const char *hvscBase);
+    inline bool tryOpenDatabase(const char *hvscBase, const char *suffix);
 
 public:
     ConsolePlayer (const char * const name);
     virtual ~ConsolePlayer() {}
 
-    int            args  (int argc, const char *argv[]);
-    bool           open  (void);
-    void           close (void);
-    bool           play  (void);
-    void           stop  (void);
-    player_state_t state (void) { return m_state; }
+    int  args  (int argc, const char *argv[]);
+    bool open  (void);
+    void close (void);
+    bool play  (void);
+    void stop  (void);
+
+    player_state_t state (void) const { return m_state; }
 };
 
 #endif // PLAYER_H
